import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.*;
import java.util.stream.Stream;

public class ac_lewin {
    static class DisjointSets {
        public static int[] createSets(int size) {
            int[] p = new int[size];
            for (int i = 0; i < size; i++)
                p[i] = i;
            return p;
        }

        public static int root(int[] p, int x) {
            return x == p[x] ? x : (p[x] = root(p, p[x]));
        }

        public static boolean unite(int[] p, int a, int b) {
            a = root(p, a);
            b = root(p, b);
            if (a != b) {
                if (Math.random() < 0.5)
                    p[a] = b;
                else
                    p[b] = a;
                return true;
            }
            return false;
        }
    }

    public boolean ok(boolean[] ss, int n, int[] a, int[] b, String c, String allowed) {
        int[] p = DisjointSets.createSets(n);
        int count = n;
        for (int i = 0; i < ss.length; i++) {
            if (ss[i] && allowed.indexOf(c.charAt(i)) != -1 && DisjointSets.unite(p, a[i], b[i])) {
                count--;
            }
        }
        return count == 1;
    }

    public int[] findPath(boolean[] ss, int n, int[] a, int[] b, int[] w, String c) {
        int m = a.length;
        int nnodes = m+2;
        List<Integer>[] graph = Stream.generate(ArrayList::new).limit(nnodes).toArray(List[]::new);
        for (int i = 0; i < m; i++) {
            for (int j = 0; j < m; j++) {
                if (ss[i] && !ss[j]) {
                    ss[i] = false;
                    ss[j] = true;
                    if (ok(ss, n, a, b, c, "RG")) graph[i].add(j);
                    if (ok(ss, n, a, b, c, "BG")) graph[j].add(i);
                    ss[i] = true;
                    ss[j] = false;
                }
            }
        }

        for (int i = 0; i < m; i++) {
            if (ss[i]) {
                ss[i] = false;
                if (ok(ss, n, a, b, c, "BG")) graph[nnodes-1].add(i);
                if (ok(ss, n, a, b, c, "RG")) graph[i].add(nnodes-2);
                ss[i] = true;
            }
        }


        int[] prev = new int[nnodes];
        int[] dist = new int[nnodes];
        Arrays.fill(prev, -1);
        Arrays.fill(dist, Integer.MAX_VALUE);
        TreeSet<Integer> ts = new TreeSet<>(Comparator.comparingInt(x -> dist[x] * (m + 10) + x));
        dist[nnodes-1] = 0;
        ts.add(nnodes-1);
        while (ts.size() > 0) {
            int f = ts.pollFirst();
            for (int nxt : graph[f]) {
                int weight = (nxt < m ? w[nxt] * (m + 10) * (ss[nxt] ? -1 : 1) : 0) + 1;
                if (dist[f] + weight < dist[nxt]) {
                    ts.remove(nxt);
                    dist[nxt] = dist[f] + weight;
                    prev[nxt] = f;
                    ts.add(nxt);
                }
            }
        }
        if (prev[nnodes-2] == -1) return null;
        int[] p = new int[nnodes];
        int cur = prev[nnodes-2];
        int id = 0;
        do {
            p[id++] = cur;
            cur = prev[cur];
        } while (cur != nnodes-1);

        return Arrays.copyOf(p, id);
    }

    public int[] findCosts(int n, int[] a, int[] b, int[] w, String c) {
        int m = a.length;
        boolean[] ss = new boolean[m];
        Arrays.fill(ss, true);
        int[] ret = new int[m];
        Arrays.fill(ret, -1);
        if (!ok(ss, n, a, b, c, "RG") || !ok(ss, n, a, b, c, "BG")) return ret;
        ret[m-1] = 0;
        for (int x : w) ret[m-1] += x;
        int idx = m-2;
        while (idx >= 0) {
            int[] x = findPath(ss, n, a, b, w, c);
            if (x == null) break;
            ret[idx] = ret[idx+1];
            for (int j : x) {
                ss[j] = !ss[j];
                if (ss[j]) {
                    ret[idx] += w[j];
                } else {
                    ret[idx] -= w[j];
                }
            }
            idx--;
        }
        return ret;
    }

    public static void main (String[] args) throws IOException {
        Scanner in = new Scanner(System.in);
        int n = in.nextInt();
        int m = in.nextInt();
        int[] a = new int[m], b = new int[m], w = new int[m];
        char[] s = new char[m];
        for (int i = 0; i < m; i++) {
            a[i] = in.nextInt()-1;
            b[i] = in.nextInt()-1;
            w[i] = in.nextInt();
            s[i] = in.next().charAt(0);
        }
        int[] res = new ac_lewin().findCosts(n, a, b, w, new String(s));
        for (int x : res) System.out.println(x);
    }
}
